from typing import Any

from ..util.langhelpers import public_factory as public_factory
from . import exc as exc, strategy_options as strategy_options
from .attributes import (
    AttributeEvent as AttributeEvent,
    InstrumentedAttribute as InstrumentedAttribute,
    Mapped as Mapped,
    QueryableAttribute as QueryableAttribute,
)
from .context import QueryContext as QueryContext
from .decl_api import (
    DeclarativeMeta as DeclarativeMeta,
    as_declarative as as_declarative,
    declarative_base as declarative_base,
    declarative_mixin as declarative_mixin,
    declared_attr as declared_attr,
    has_inherited_table as has_inherited_table,
    registry as registry,
    synonym_for as synonym_for,
)
from .descriptor_props import CompositeProperty as CompositeProperty, SynonymProperty as SynonymProperty
from .dynamic import AppenderQuery as AppenderQuery
from .events import (
    AttributeEvents as AttributeEvents,
    InstanceEvents as InstanceEvents,
    InstrumentationEvents as InstrumentationEvents,
    MapperEvents as MapperEvents,
    QueryEvents as QueryEvents,
    SessionEvents as SessionEvents,
)
from .identity import IdentityMap as IdentityMap
from .instrumentation import ClassManager as ClassManager
from .interfaces import (
    EXT_CONTINUE as EXT_CONTINUE,
    EXT_SKIP as EXT_SKIP,
    EXT_STOP as EXT_STOP,
    MANYTOMANY as MANYTOMANY,
    MANYTOONE as MANYTOONE,
    NOT_EXTENSION as NOT_EXTENSION,
    ONETOMANY as ONETOMANY,
    InspectionAttr as InspectionAttr,
    InspectionAttrInfo as InspectionAttrInfo,
    MapperProperty as MapperProperty,
    PropComparator as PropComparator,
    UserDefinedOption as UserDefinedOption,
)
from .loading import merge_frozen_result as merge_frozen_result, merge_result as merge_result
from .mapper import (
    Mapper as Mapper,
    class_mapper as class_mapper,
    configure_mappers as configure_mappers,
    reconstructor as reconstructor,
    validates as validates,
)
from .properties import ColumnProperty as ColumnProperty
from .query import AliasOption as AliasOption, FromStatement as FromStatement, Query as Query
from .relationships import RelationshipProperty as RelationshipProperty, foreign as foreign, remote as remote
from .scoping import scoped_session as scoped_session
from .session import (
    ORMExecuteState as ORMExecuteState,
    Session as Session,
    SessionTransaction as SessionTransaction,
    close_all_sessions as close_all_sessions,
    make_transient as make_transient,
    make_transient_to_detached as make_transient_to_detached,
    object_session as object_session,
    sessionmaker as sessionmaker,
)
from .state import AttributeState as AttributeState, InstanceState as InstanceState
from .strategy_options import Load as Load
from .unitofwork import UOWTransaction as UOWTransaction
from .util import (
    Bundle as Bundle,
    CascadeOptions as CascadeOptions,
    LoaderCriteriaOption as LoaderCriteriaOption,
    aliased as aliased,
    join as join,
    object_mapper as object_mapper,
    outerjoin as outerjoin,
    polymorphic_union as polymorphic_union,
    was_deleted as was_deleted,
    with_parent as with_parent,
    with_polymorphic as with_polymorphic,
)

def create_session(bind: Any | None = ..., **kwargs): ...

with_loader_criteria: Any
relationship: Any

def relation(*arg, **kw): ...
def dynamic_loader(argument, **kw): ...

column_property: Any
composite: Any

def backref(name, **kwargs): ...
def deferred(*columns, **kw): ...
def query_expression(default_expr=...): ...

mapper: Any
synonym: Any

def clear_mappers() -> None: ...

joinedload: Any
contains_eager: Any
defer: Any
undefer: Any
undefer_group: Any
with_expression: Any
load_only: Any
lazyload: Any
subqueryload: Any
selectinload: Any
immediateload: Any
noload: Any
raiseload: Any
defaultload: Any
selectin_polymorphic: Any

def eagerload(*args, **kwargs): ...

contains_alias: Any



from collections import defaultdict as defaultdict
from contextlib import contextmanager as contextmanager
from functools import partial as partial, update_wrapper as update_wrapper

from ._collections import (
    EMPTY_DICT as EMPTY_DICT,
    EMPTY_SET as EMPTY_SET,
    FacadeDict as FacadeDict,
    IdentitySet as IdentitySet,
    ImmutableContainer as ImmutableContainer,
    ImmutableProperties as ImmutableProperties,
    LRUCache as LRUCache,
    OrderedDict as OrderedDict,
    OrderedIdentitySet as OrderedIdentitySet,
    OrderedProperties as OrderedProperties,
    OrderedSet as OrderedSet,
    PopulateDict as PopulateDict,
    Properties as Properties,
    ScopedRegistry as ScopedRegistry,
    ThreadLocalRegistry as ThreadLocalRegistry,
    UniqueAppender as UniqueAppender,
    WeakPopulateDict as WeakPopulateDict,
    WeakSequence as WeakSequence,
    coerce_generator_arg as coerce_generator_arg,
    coerce_to_immutabledict as coerce_to_immutabledict,
    collections_abc as collections_abc,
    column_dict as column_dict,
    column_set as column_set,
    flatten_iterator as flatten_iterator,
    has_dupes as has_dupes,
    has_intersection as has_intersection,
    immutabledict as immutabledict,
    ordered_column_set as ordered_column_set,
    sort_dictionary as sort_dictionary,
    to_column_set as to_column_set,
    to_list as to_list,
    to_set as to_set,
    unique_list as unique_list,
    update_copy as update_copy,
)
from ._preloaded import preload_module as preload_module, preloaded as preloaded
from .compat import (
    ABC as ABC,
    TYPE_CHECKING as TYPE_CHECKING,
    StringIO as StringIO,
    arm as arm,
    b as b,
    b64decode as b64decode,
    b64encode as b64encode,
    binary_type as binary_type,
    binary_types as binary_types,
    byte_buffer as byte_buffer,
    callable as callable,
    cmp as cmp,
    cpython as cpython,
    dataclass_fields as dataclass_fields,
    decode_backslashreplace as decode_backslashreplace,
    dottedgetter as dottedgetter,
    has_refcount_gc as has_refcount_gc,
    inspect_getfullargspec as inspect_getfullargspec,
    int_types as int_types,
    iterbytes as iterbytes,
    itertools_filter as itertools_filter,
    itertools_filterfalse as itertools_filterfalse,
    local_dataclass_fields as local_dataclass_fields,
    namedtuple as namedtuple,
    next as next,
    nullcontext as nullcontext,
    osx as osx,
    parse_qsl as parse_qsl,
    perf_counter as perf_counter,
    pickle as pickle,
    print_ as print_,
    py2k as py2k,
    py3k as py3k,
    py37 as py37,
    py38 as py38,
    py39 as py39,
    pypy as pypy,
    quote_plus as quote_plus,
    raise_ as raise_,
    raise_from_cause as raise_from_cause,
    reduce as reduce,
    reraise as reraise,
    string_types as string_types,
    text_type as text_type,
    threading as threading,
    timezone as timezone,
    u as u,
    ue as ue,
    unquote as unquote,
    unquote_plus as unquote_plus,
    win32 as win32,
    with_metaclass as with_metaclass,
    zip_longest as zip_longest,
)
from .concurrency import (
    asyncio as asyncio,
    await_fallback as await_fallback,
    await_only as await_only,
    greenlet_spawn as greenlet_spawn,
    is_exit_exception as is_exit_exception,
)
from .deprecations import (
    SQLALCHEMY_WARN_20 as SQLALCHEMY_WARN_20,
    deprecated as deprecated,
    deprecated_20 as deprecated_20,
    deprecated_20_cls as deprecated_20_cls,
    deprecated_cls as deprecated_cls,
    deprecated_params as deprecated_params,
    inject_docstring_text as inject_docstring_text,
    moved_20 as moved_20,
    warn_deprecated as warn_deprecated,
    warn_deprecated_20 as warn_deprecated_20,
)
from .langhelpers import (
    EnsureKWArgType as EnsureKWArgType,
    HasMemoized as HasMemoized,
    MemoizedSlots as MemoizedSlots,
    NoneType as NoneType,
    PluginLoader as PluginLoader,
    add_parameter_text as add_parameter_text,
    as_interface as as_interface,
    asbool as asbool,
    asint as asint,
    assert_arg_type as assert_arg_type,
    attrsetter as attrsetter,
    bool_or_str as bool_or_str,
    chop_traceback as chop_traceback,
    class_hierarchy as class_hierarchy,
    classproperty as classproperty,
    clsname_as_plain_name as clsname_as_plain_name,
    coerce_kw_type as coerce_kw_type,
    constructor_copy as constructor_copy,
    constructor_key as constructor_key,
    counter as counter,
    create_proxy_methods as create_proxy_methods,
    decode_slice as decode_slice,
    decorator as decorator,
    dictlike_iteritems as dictlike_iteritems,
    duck_type_collection as duck_type_collection,
    ellipses_string as ellipses_string,
    format_argspec_init as format_argspec_init,
    format_argspec_plus as format_argspec_plus,
    generic_repr as generic_repr,
    get_callable_argspec as get_callable_argspec,
    get_cls_kwargs as get_cls_kwargs,
    get_func_kwargs as get_func_kwargs,
    getargspec_init as getargspec_init,
    has_compiled_ext as has_compiled_ext,
    hybridmethod as hybridmethod,
    hybridproperty as hybridproperty,
    iterate_attributes as iterate_attributes,
    map_bits as map_bits,
    md5_hex as md5_hex,
    memoized_instancemethod as memoized_instancemethod,
    memoized_property as memoized_property,
    method_is_overridden as method_is_overridden,
    methods_equivalent as methods_equivalent,
    monkeypatch_proxied_specials as monkeypatch_proxied_specials,
    only_once as only_once,
    portable_instancemethod as portable_instancemethod,
    quoted_token_parser as quoted_token_parser,
    safe_reraise as safe_reraise,
    set_creation_order as set_creation_order,
    string_or_unprintable as string_or_unprintable,
    symbol as symbol,
    unbound_method_to_callable as unbound_method_to_callable,
    walk_subclasses as walk_subclasses,
    warn as warn,
    warn_exception as warn_exception,
    warn_limited as warn_limited,
    wrap_callable as wrap_callable,
)



from typing import Any

class ReflectedState:
    columns: Any
    table_options: Any
    table_name: Any
    keys: Any
    fk_constraints: Any
    ck_constraints: Any
    def __init__(self) -> None: ...

class MySQLTableDefinitionParser:
    logger: Any
    dialect: Any
    preparer: Any
    def __init__(self, dialect, preparer) -> None: ...
    def parse(self, show_create, charset): ...



from typing import Any

def ordering_list(attr, count_from: Any | None = ..., **kw): ...

class OrderingList(list[Any]):
    ordering_attr: Any
    ordering_func: Any
    reorder_on_append: Any
    def __init__(
        self, ordering_attr: Any | None = ..., ordering_func: Any | None = ..., reorder_on_append: bool = ...
    ) -> None: ...
    def reorder(self) -> None: ...
    def append(self, entity) -> None: ...
    def insert(self, index, entity) -> None: ...
    def remove(self, entity) -> None: ...
    def pop(self, index: int = ...): ...  # type: ignore[override]
    def __setitem__(self, index, entity) -> None: ...
    def __delitem__(self, index) -> None: ...
    def __setslice__(self, start, end, values) -> None: ...
    def __delslice__(self, start, end) -> None: ...
    def __reduce__(self): ...



from collections.abc import Callable
from typing import Any, Type as TypingType

MypyFile = Any  # from mypy.nodes
AttributeContext = Any  # from mypy.plugin
ClassDefContext = Any  # from mypy.plugin
DynamicClassDefContext = Any  # from mypy.plugin
Plugin = Any  # from mypy.plugin
Type = Any  # from mypy.types

class SQLAlchemyPlugin(Plugin):
    def get_dynamic_class_hook(self, fullname: str) -> Callable[[DynamicClassDefContext], None] | None: ...
    def get_customize_class_mro_hook(self, fullname: str) -> Callable[[ClassDefContext], None] | None: ...
    def get_class_decorator_hook(self, fullname: str) -> Callable[[ClassDefContext], None] | None: ...
    def get_metaclass_hook(self, fullname: str) -> Callable[[ClassDefContext], None] | None: ...
    def get_base_class_hook(self, fullname: str) -> Callable[[ClassDefContext], None] | None: ...
    def get_attribute_hook(self, fullname: str) -> Callable[[AttributeContext], Type] | None: ...
    def get_additional_deps(self, file: MypyFile) -> list[tuple[int, str, int]]: ...

def plugin(version: str) -> TypingType[SQLAlchemyPlugin]: ...



from collections.abc import Callable
from typing import Any, Generic, TypeVar, overload

from . import compat

_R = TypeVar("_R")
_T = TypeVar("_T")

def md5_hex(x): ...

class safe_reraise:
    warn_only: Any
    def __init__(self, warn_only: bool = ...) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, type_, value, traceback) -> None: ...

def walk_subclasses(cls) -> None: ...
def string_or_unprintable(element): ...
def clsname_as_plain_name(cls): ...
def method_is_overridden(instance_or_cls, against_method): ...
def decode_slice(slc): ...
def map_bits(fn, n) -> None: ...
def decorator(target): ...
def public_factory(target, location, class_location: Any | None = ...): ...

class PluginLoader:
    group: Any
    impls: Any
    auto_fn: Any
    def __init__(self, group, auto_fn: Any | None = ...) -> None: ...
    def clear(self) -> None: ...
    def load(self, name): ...
    def register(self, name, modulepath, objname): ...

def get_cls_kwargs(cls, _set: Any | None = ...): ...
def get_func_kwargs(func): ...
def get_callable_argspec(fn, no_self: bool = ..., _is_init: bool = ...): ...
def format_argspec_plus(fn, grouped: bool = ...): ...
def format_argspec_init(method, grouped: bool = ...): ...
def create_proxy_methods(
    target_cls, target_cls_sphinx_name, proxy_cls_sphinx_name, classmethods=..., methods=..., attributes=...
): ...
def getargspec_init(method): ...
def unbound_method_to_callable(func_or_cls): ...
def generic_repr(obj, additional_kw=..., to_inspect: Any | None = ..., omit_kwarg=...): ...

class portable_instancemethod:
    target: Any
    name: Any
    kwargs: Any
    def __init__(self, meth, kwargs=...) -> None: ...
    def __call__(self, *arg, **kw): ...

def class_hierarchy(cls): ...
def iterate_attributes(cls) -> None: ...
def monkeypatch_proxied_specials(
    into_cls, from_cls, skip: Any | None = ..., only: Any | None = ..., name: str = ..., from_instance: Any | None = ...
) -> None: ...
def methods_equivalent(meth1, meth2): ...
def as_interface(obj, cls: Any | None = ..., methods: Any | None = ..., required: Any | None = ...): ...

class memoized_property(Generic[_R]):
    fget: Callable[..., _R]
    __doc__: str
    __name__: str
    def __init__(self, fget: Callable[..., _R], doc: str | None = ...) -> None: ...
    @overload
    def __get__(self: _T, obj: None, cls: object) -> _T: ...
    @overload
    def __get__(self, obj: object, cls: object) -> _R: ...
    @classmethod
    def reset(cls, obj: object, name: str) -> None: ...

def memoized_instancemethod(fn): ...

class HasMemoized:
    class memoized_attribute(Generic[_R]):
        fget: Callable[..., _R]
        __doc__: str
        __name__: str
        def __init__(self, fget: Callable[..., _R], doc: str | None = ...) -> None: ...
        @overload
        def __get__(self: _T, obj: None, cls: object) -> _T: ...
        @overload
        def __get__(self, obj: object, cls: object) -> _R: ...
    @classmethod
    def memoized_instancemethod(cls, fn): ...

class MemoizedSlots:
    def __getattr__(self, key): ...

def asbool(obj): ...
def bool_or_str(*text): ...
def asint(value): ...
def coerce_kw_type(kw, key, type_, flexi_bool: bool = ..., dest: Any | None = ...) -> None: ...
def constructor_key(obj, cls): ...
def constructor_copy(obj, cls, *args, **kw): ...
def counter(): ...
def duck_type_collection(specimen, default: Any | None = ...): ...
def assert_arg_type(arg, argtype, name): ...
def dictlike_iteritems(dictlike): ...

class classproperty(property):
    __doc__: Any
    def __init__(self, fget, *arg, **kw) -> None: ...
    def __get__(self, self_, cls): ...

class hybridproperty(Generic[_R]):
    func: Callable[..., _R]
    clslevel: Callable[..., _R]
    def __init__(self, func: Callable[..., _R]) -> None: ...
    @overload
    def __get__(self, instance: None, owner: Any) -> _R: ...
    @overload
    def __get__(self, instance: object, owner: object) -> _R: ...
    def classlevel(self: _T, func: Callable[..., _R]) -> _T: ...

class hybridmethod:
    func: Any
    clslevel: Any
    def __init__(self, func) -> None: ...
    def __get__(self, instance, owner): ...
    def classlevel(self, func): ...

class _symbol(int):
    def __new__(cls, name, doc: Any | None = ..., canonical: Any | None = ...): ...
    def __reduce__(self): ...

class symbol:
    symbols: Any
    def __new__(cls, name, doc: Any | None = ..., canonical: Any | None = ...): ...
    @classmethod
    def parse_user_argument(cls, arg, choices, name, resolve_symbol_names: bool = ...): ...

def set_creation_order(instance) -> None: ...
def warn_exception(func, *args, **kwargs): ...
def ellipses_string(value, len_: int = ...): ...

class _hash_limit_string(compat.text_type):
    def __new__(cls, value, num, args): ...
    def __hash__(self): ...
    def __eq__(self, other): ...

def warn(msg, code: Any | None = ...) -> None: ...
def warn_limited(msg, args) -> None: ...
def only_once(fn, retry_on_exception): ...
def chop_traceback(tb, exclude_prefix=..., exclude_suffix=...): ...

NoneType: Any

def attrsetter(attrname): ...

class EnsureKWArgType(type):
    def __init__(cls, clsname, bases, clsdict) -> None: ...

def wrap_callable(wrapper, fn): ...
def quoted_token_parser(value): ...
def add_parameter_text(params, text): ...
def inject_docstring_text(doctext, injecttext, pos): ...
def inject_param_text(doctext, inject_params): ...
def repr_tuple_names(names): ...
def has_compiled_ext(): ...



from typing import Any

requirements: Any
db: Any
db_url: Any
db_opts: Any
file_config: Any
test_schema: Any
test_schema_2: Any
any_async: bool
ident: str

def combinations(*comb, **kw): ...
def combinations_list(arg_iterable, **kw): ...
def fixture(*arg, **kw): ...
def get_current_test_name(): ...
def mark_base_test_class(): ...

class Config:
    db: Any
    db_opts: Any
    options: Any
    file_config: Any
    test_schema: str
    test_schema_2: str
    is_async: Any
    def __init__(self, db, db_opts, options, file_config) -> None: ...
    @classmethod
    def register(cls, db, db_opts, options, file_config): ...
    @classmethod
    def set_as_current(cls, config, namespace) -> None: ...
    @classmethod
    def push_engine(cls, db, namespace) -> None: ...
    @classmethod
    def push(cls, config, namespace) -> None: ...
    @classmethod
    def pop(cls, namespace) -> None: ...
    @classmethod
    def reset(cls, namespace) -> None: ...
    @classmethod
    def all_configs(cls): ...
    @classmethod
    def all_dbs(cls) -> None: ...
    def skip_test(self, msg) -> None: ...

def skip_test(msg) -> None: ...
def async_test(fn): ...



from typing import Any

from ...util import memoized_property
from .base import BIT, MySQLCompiler, MySQLDialect, MySQLIdentifierPreparer

class MySQLCompiler_mysqlconnector(MySQLCompiler):
    def visit_mod_binary(self, binary, operator, **kw): ...
    def post_process_text(self, text): ...
    def escape_literal_column(self, text): ...

class MySQLIdentifierPreparer_mysqlconnector(MySQLIdentifierPreparer): ...

class _myconnpyBIT(BIT):
    def result_processor(self, dialect, coltype) -> None: ...

class MySQLDialect_mysqlconnector(MySQLDialect):
    driver: str
    supports_statement_cache: bool
    supports_unicode_binds: bool
    supports_sane_rowcount: bool
    supports_sane_multi_rowcount: bool
    supports_native_decimal: bool
    default_paramstyle: str
    statement_compiler: Any
    preparer: Any
    colspecs: Any
    def __init__(self, *arg, **kw) -> None: ...
    @property
    def description_encoding(self): ...
    @memoized_property
    def supports_unicode_statements(self): ...
    @classmethod
    def dbapi(cls): ...
    def do_ping(self, dbapi_connection): ...
    def create_connect_args(self, url): ...
    def is_disconnect(self, e, connection, cursor): ...

dialect = MySQLDialect_mysqlconnector



from typing import Any

from .. import event

class InstrumentationEvents(event.Events):
    def class_instrument(self, cls) -> None: ...
    def class_uninstrument(self, cls) -> None: ...
    def attribute_instrument(self, cls, key, inst) -> None: ...

class _InstrumentationEventsHold:
    class_: Any
    def __init__(self, class_) -> None: ...
    dispatch: Any

class InstanceEvents(event.Events):
    def first_init(self, manager, cls) -> None: ...
    def init(self, target, args, kwargs) -> None: ...
    def init_failure(self, target, args, kwargs) -> None: ...
    def load(self, target, context) -> None: ...
    def refresh(self, target, context, attrs) -> None: ...
    def refresh_flush(self, target, flush_context, attrs) -> None: ...
    def expire(self, target, attrs) -> None: ...
    def pickle(self, target, state_dict) -> None: ...
    def unpickle(self, target, state_dict) -> None: ...

class _EventsHold(event.RefCollection):
    class_: Any
    def __init__(self, class_) -> None: ...
    class HoldEvents: ...
    def remove(self, event_key) -> None: ...
    @classmethod
    def populate(cls, class_, subject) -> None: ...

class _InstanceEventsHold(_EventsHold):
    all_holds: Any
    def resolve(self, class_): ...
    class HoldInstanceEvents(_EventsHold.HoldEvents, InstanceEvents): ...
    dispatch: Any

class MapperEvents(event.Events):
    def instrument_class(self, mapper, class_) -> None: ...
    def before_mapper_configured(self, mapper, class_) -> None: ...
    def mapper_configured(self, mapper, class_) -> None: ...
    def before_configured(self) -> None: ...
    def after_configured(self) -> None: ...
    def before_insert(self, mapper, connection, target) -> None: ...
    def after_insert(self, mapper, connection, target) -> None: ...
    def before_update(self, mapper, connection, target) -> None: ...
    def after_update(self, mapper, connection, target) -> None: ...
    def before_delete(self, mapper, connection, target) -> None: ...
    def after_delete(self, mapper, connection, target) -> None: ...

class _MapperEventsHold(_EventsHold):
    all_holds: Any
    def resolve(self, class_): ...
    class HoldMapperEvents(_EventsHold.HoldEvents, MapperEvents): ...
    dispatch: Any

class SessionEvents(event.Events):
    def do_orm_execute(self, orm_execute_state) -> None: ...
    def after_transaction_create(self, session, transaction) -> None: ...
    def after_transaction_end(self, session, transaction) -> None: ...
    def before_commit(self, session) -> None: ...
    def after_commit(self, session) -> None: ...
    def after_rollback(self, session) -> None: ...
    def after_soft_rollback(self, session, previous_transaction) -> None: ...
    def before_flush(self, session, flush_context, instances) -> None: ...
    def after_flush(self, session, flush_context) -> None: ...
    def after_flush_postexec(self, session, flush_context) -> None: ...
    def after_begin(self, session, transaction, connection) -> None: ...
    def before_attach(self, session, instance) -> None: ...
    def after_attach(self, session, instance) -> None: ...
    def after_bulk_update(self, update_context) -> None: ...
    def after_bulk_delete(self, delete_context) -> None: ...
    def transient_to_pending(self, session, instance) -> None: ...
    def pending_to_transient(self, session, instance) -> None: ...
    def persistent_to_transient(self, session, instance) -> None: ...
    def pending_to_persistent(self, session, instance) -> None: ...
    def detached_to_persistent(self, session, instance) -> None: ...
    def loaded_as_persistent(self, session, instance) -> None: ...
    def persistent_to_deleted(self, session, instance) -> None: ...
    def deleted_to_persistent(self, session, instance) -> None: ...
    def deleted_to_detached(self, session, instance) -> None: ...
    def persistent_to_detached(self, session, instance) -> None: ...

class AttributeEvents(event.Events):
    def append(self, target, value, initiator) -> None: ...
    def append_wo_mutation(self, target, value, initiator) -> None: ...
    def bulk_replace(self, target, values, initiator) -> None: ...
    def remove(self, target, value, initiator) -> None: ...
    def set(self, target, value, oldvalue, initiator) -> None: ...
    def init_scalar(self, target, value, dict_) -> None: ...
    def init_collection(self, target, collection, collection_adapter) -> None: ...
    def dispose_collection(self, target, collection, collection_adapter) -> None: ...
    def modified(self, target, initiator) -> None: ...

class QueryEvents(event.Events):
    def before_compile(self, query) -> None: ...
    def before_compile_update(self, query, update_context) -> None: ...
    def before_compile_delete(self, query, delete_context) -> None: ...



from typing import Any

def non_refcount_gc_collect(*args) -> None: ...  # only present on Python implementations with non-refcount gc
def gc_collect(generation: int = ...) -> None: ...
def lazy_gc() -> None: ...
def picklers(): ...
def random_choices(population, k: int = ...): ...
def round_decimal(value, prec): ...

class RandomSet(set[Any]):
    def __iter__(self): ...
    def pop(self): ...
    def union(self, other): ...
    def difference(self, other): ...
    def intersection(self, other): ...
    def copy(self): ...

def conforms_partial_ordering(tuples, sorted_elements): ...
def all_partial_orderings(tuples, elements): ...
def function_named(fn, name): ...
def run_as_contextmanager(ctx, fn, *arg, **kw): ...
def rowset(results): ...
def fail(msg) -> None: ...
def provide_metadata(fn, *args, **kw): ...
def flag_combinations(*combinations): ...
def lambda_combinations(lambda_arg_sets, **kw): ...
def resolve_lambda(__fn, **kw): ...
def metadata_fixture(ddl: str = ...): ...
def force_drop_names(*names): ...

class adict(dict[Any, Any]):
    def __getattribute__(self, key): ...
    def __call__(self, *keys): ...
    get_all: Any

def drop_all_tables_from_metadata(metadata, engine_or_connection) -> None: ...
def drop_all_tables(engine, inspector, schema: Any | None = ..., include_names: Any | None = ...) -> None: ...
def teardown_events(event_cls): ...



from typing import Any

from ..util import memoized_property

class URL:
    def __new__(cls, *arg, **kw): ...
    @classmethod
    def create(
        cls,
        drivername,
        username: Any | None = ...,
        password: Any | None = ...,
        host: Any | None = ...,
        port: Any | None = ...,
        database: Any | None = ...,
        query=...,
    ): ...
    def set(
        self,
        drivername: Any | None = ...,
        username: Any | None = ...,
        password: Any | None = ...,
        host: Any | None = ...,
        port: Any | None = ...,
        database: Any | None = ...,
        query: Any | None = ...,
    ): ...
    def update_query_string(self, query_string, append: bool = ...): ...
    def update_query_pairs(self, key_value_pairs, append: bool = ...): ...
    def update_query_dict(self, query_parameters, append: bool = ...): ...
    def difference_update_query(self, names): ...
    @memoized_property
    def normalized_query(self): ...
    def __to_string__(self, hide_password: bool = ...): ...
    def render_as_string(self, hide_password: bool = ...): ...
    def __copy__(self): ...
    def __deepcopy__(self, memo): ...
    def __hash__(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def get_backend_name(self): ...
    def get_driver_name(self): ...
    def get_dialect(self): ...
    def translate_connect_args(self, names: Any | None = ..., **kw): ...

def make_url(name_or_url): ...



from typing import Any

from .base import ProxyComparable, StartableContext

def create_async_engine(*arg, **kw): ...

class AsyncConnectable: ...

class AsyncConnection(ProxyComparable, StartableContext, AsyncConnectable):
    engine: Any
    sync_engine: Any
    sync_connection: Any
    def __init__(self, async_engine, sync_connection: Any | None = ...) -> None: ...
    async def start(self, is_ctxmanager: bool = ...): ...
    @property
    def connection(self) -> None: ...
    async def get_raw_connection(self): ...
    @property
    def info(self): ...
    def begin(self): ...
    def begin_nested(self): ...
    async def invalidate(self, exception: Any | None = ...): ...
    async def get_isolation_level(self): ...
    async def set_isolation_level(self): ...
    def in_transaction(self): ...
    def in_nested_transaction(self): ...
    def get_transaction(self): ...
    def get_nested_transaction(self): ...
    async def execution_options(self, **opt): ...
    async def commit(self) -> None: ...
    async def rollback(self) -> None: ...
    async def close(self) -> None: ...
    async def exec_driver_sql(self, statement, parameters: Any | None = ..., execution_options=...): ...
    async def stream(self, statement, parameters: Any | None = ..., execution_options=...): ...
    async def execute(self, statement, parameters: Any | None = ..., execution_options=...): ...
    async def scalar(self, statement, parameters: Any | None = ..., execution_options=...): ...
    async def scalars(self, statement, parameters: Any | None = ..., execution_options=...): ...
    async def stream_scalars(self, statement, parameters: Any | None = ..., execution_options=...): ...
    async def run_sync(self, fn, *arg, **kw): ...
    def __await__(self): ...
    async def __aexit__(self, type_, value, traceback) -> None: ...
    # proxied from Connection
    dialect: Any
    @property
    def closed(self): ...
    @property
    def invalidated(self): ...
    @property
    def default_isolation_level(self): ...

class AsyncEngine(ProxyComparable, AsyncConnectable):
    class _trans_ctx(StartableContext):
        conn: Any
        def __init__(self, conn) -> None: ...
        transaction: Any
        async def start(self, is_ctxmanager: bool = ...): ...
        async def __aexit__(self, type_, value, traceback) -> None: ...
    sync_engine: Any
    def __init__(self, sync_engine) -> None: ...
    def begin(self): ...
    def connect(self): ...
    async def raw_connection(self): ...
    def execution_options(self, **opt): ...
    async def dispose(self): ...
    # proxied from Engine
    url: Any
    pool: Any
    dialect: Any
    echo: Any
    @property
    def engine(self): ...
    @property
    def name(self): ...
    @property
    def driver(self): ...
    def clear_compiled_cache(self) -> None: ...
    def update_execution_options(self, **opt) -> None: ...
    def get_execution_options(self): ...

class AsyncTransaction(ProxyComparable, StartableContext):
    connection: Any
    sync_transaction: Any
    nested: Any
    def __init__(self, connection, nested: bool = ...) -> None: ...
    @property
    def is_valid(self): ...
    @property
    def is_active(self): ...
    async def close(self) -> None: ...
    async def rollback(self) -> None: ...
    async def commit(self) -> None: ...
    async def start(self, is_ctxmanager: bool = ...): ...
    async def __aexit__(self, type_, value, traceback) -> None: ...



from typing import Any

from ...util import symbol

ClassDef = Any  # from mypy.nodes
Expression = Any  # from mypy.nodes
MemberExpr = Any  # from mypy.nodes
NameExpr = Any  # from mypy.nodes
SymbolNode = Any  # from mypy.nodes
TypeInfo = Any  # from mypy.nodes
StrExpr = Any  # from mypy.nodes
SemanticAnalyzerPluginInterface = Any  # from mypy.plugin
UnboundType = Any  # from mypy.types

COLUMN: symbol
RELATIONSHIP: symbol
REGISTRY: symbol
COLUMN_PROPERTY: symbol
TYPEENGINE: symbol
MAPPED: symbol
DECLARATIVE_BASE: symbol
DECLARATIVE_META: symbol
MAPPED_DECORATOR: symbol
SYNONYM_PROPERTY: symbol
COMPOSITE_PROPERTY: symbol
DECLARED_ATTR: symbol
MAPPER_PROPERTY: symbol
AS_DECLARATIVE: symbol
AS_DECLARATIVE_BASE: symbol
DECLARATIVE_MIXIN: symbol
QUERY_EXPRESSION: symbol

def has_base_type_id(info: TypeInfo, type_id: int) -> bool: ...
def mro_has_id(mro: list[TypeInfo], type_id: int) -> bool: ...
def type_id_for_unbound_type(type_: UnboundType, cls: ClassDef, api: SemanticAnalyzerPluginInterface) -> int | None: ...
def type_id_for_callee(callee: Expression) -> int | None: ...
def type_id_for_named_node(node: NameExpr | MemberExpr | SymbolNode) -> int | None: ...
def type_id_for_fullname(fullname: str) -> int | None: ...



from .. import event as event

class ConnectionEvents(event.Events):
    def before_execute(self, conn, clauseelement, multiparams, params, execution_options) -> None: ...
    def after_execute(self, conn, clauseelement, multiparams, params, execution_options, result) -> None: ...
    def before_cursor_execute(self, conn, cursor, statement, parameters, context, executemany) -> None: ...
    def after_cursor_execute(self, conn, cursor, statement, parameters, context, executemany) -> None: ...
    def handle_error(self, exception_context) -> None: ...
    def engine_connect(self, conn, branch) -> None: ...
    def set_connection_execution_options(self, conn, opts) -> None: ...
    def set_engine_execution_options(self, engine, opts) -> None: ...
    def engine_disposed(self, engine) -> None: ...
    def begin(self, conn) -> None: ...
    def rollback(self, conn) -> None: ...
    def commit(self, conn) -> None: ...
    def savepoint(self, conn, name) -> None: ...
    def rollback_savepoint(self, conn, name, context) -> None: ...
    def release_savepoint(self, conn, name, context) -> None: ...
    def begin_twophase(self, conn, xid) -> None: ...
    def prepare_twophase(self, conn, xid) -> None: ...
    def rollback_twophase(self, conn, xid, is_prepared) -> None: ...
    def commit_twophase(self, conn, xid, is_prepared) -> None: ...

class DialectEvents(event.Events):
    def do_connect(self, dialect, conn_rec, cargs, cparams) -> None: ...
    def do_executemany(self, cursor, statement, parameters, context) -> None: ...
    def do_execute_no_params(self, cursor, statement, context) -> None: ...
    def do_execute(self, cursor, statement, parameters, context) -> None: ...
    def do_setinputsizes(self, inputsizes, cursor, statement, parameters, context) -> None: ...



from typing import Any

from . import roles

elements: Any
lambdas: Any
schema: Any
selectable: Any
sqltypes: Any
traversals: Any

def expect(role, element, apply_propagate_attrs: Any | None = ..., argname: Any | None = ..., post_inspect: bool = ..., **kw): ...
def expect_as_key(role, element, **kw): ...
def expect_col_expression_collection(role, expressions) -> None: ...

class RoleImpl:
    name: Any
    def __init__(self, role_class) -> None: ...

class _Deannotate: ...
class _StringOnly: ...
class _ReturnsStringKey: ...
class _ColumnCoercions: ...
class _NoTextCoercion: ...
class _CoerceLiterals: ...
class LiteralValueImpl(RoleImpl): ...
class _SelectIsNotFrom: ...
class HasCacheKeyImpl(RoleImpl): ...
class ExecutableOptionImpl(RoleImpl): ...
class ExpressionElementImpl(_ColumnCoercions, RoleImpl): ...
class BinaryElementImpl(ExpressionElementImpl, RoleImpl): ...
class InElementImpl(RoleImpl): ...
class OnClauseImpl(_CoerceLiterals, _ColumnCoercions, RoleImpl): ...
class WhereHavingImpl(_CoerceLiterals, _ColumnCoercions, RoleImpl): ...
class StatementOptionImpl(_CoerceLiterals, RoleImpl): ...
class ColumnArgumentImpl(_NoTextCoercion, RoleImpl): ...
class ColumnArgumentOrKeyImpl(_ReturnsStringKey, RoleImpl): ...
class StrAsPlainColumnImpl(_CoerceLiterals, RoleImpl): ...
class ByOfImpl(_CoerceLiterals, _ColumnCoercions, RoleImpl, roles.ByOfRole): ...
class OrderByImpl(ByOfImpl, RoleImpl): ...
class GroupByImpl(ByOfImpl, RoleImpl): ...
class DMLColumnImpl(_ReturnsStringKey, RoleImpl): ...
class ConstExprImpl(RoleImpl): ...
class TruncatedLabelImpl(_StringOnly, RoleImpl): ...
class DDLExpressionImpl(_Deannotate, _CoerceLiterals, RoleImpl): ...
class DDLConstraintColumnImpl(_Deannotate, _ReturnsStringKey, RoleImpl): ...
class DDLReferredColumnImpl(DDLConstraintColumnImpl): ...
class LimitOffsetImpl(RoleImpl): ...
class LabeledColumnExprImpl(ExpressionElementImpl): ...
class ColumnsClauseImpl(_SelectIsNotFrom, _CoerceLiterals, RoleImpl): ...
class ReturnsRowsImpl(RoleImpl): ...
class StatementImpl(_CoerceLiterals, RoleImpl): ...
class SelectStatementImpl(_NoTextCoercion, RoleImpl): ...
class HasCTEImpl(ReturnsRowsImpl): ...
class IsCTEImpl(RoleImpl): ...
class JoinTargetImpl(RoleImpl): ...
class FromClauseImpl(_SelectIsNotFrom, _NoTextCoercion, RoleImpl): ...
class StrictFromClauseImpl(FromClauseImpl): ...
class AnonymizedFromClauseImpl(StrictFromClauseImpl): ...
class DMLTableImpl(_SelectIsNotFrom, _NoTextCoercion, RoleImpl): ...
class DMLSelectImpl(_NoTextCoercion, RoleImpl): ...
class CompoundElementImpl(_NoTextCoercion, RoleImpl): ...

cls: Any
name: Any
impl: Any



from typing import Any

from sqlalchemy import types as sqltypes
from sqlalchemy.connectors.pyodbc import PyODBCConnector
from sqlalchemy.dialects.sybase.base import SybaseDialect, SybaseExecutionContext

class _SybNumeric_pyodbc(sqltypes.Numeric):
    def bind_processor(self, dialect): ...

class SybaseExecutionContext_pyodbc(SybaseExecutionContext):
    def set_ddl_autocommit(self, connection, value) -> None: ...

class SybaseDialect_pyodbc(PyODBCConnector, SybaseDialect):
    supports_statement_cache: bool
    colspecs: Any
    @classmethod
    def dbapi(cls): ...

dialect = SybaseDialect_pyodbc



from typing import Any

from ..util import memoized_property

class ScopedSessionMixin:
    def __call__(self, **kw): ...
    def configure(self, **kwargs) -> None: ...

class scoped_session(ScopedSessionMixin):
    session_factory: Any
    registry: Any
    def __init__(self, session_factory, scopefunc: Any | None = ...) -> None: ...
    def remove(self) -> None: ...
    def query_property(self, query_cls: Any | None = ...): ...
    # dynamically proxied from class Session
    bind: Any
    identity_map: Any
    autoflush: Any
    autocommit: bool
    @property
    def dirty(self): ...
    @property
    def deleted(self): ...
    @property
    def new(self): ...
    @property
    def is_active(self): ...
    @property
    def no_autoflush(self) -> None: ...
    @memoized_property
    def info(self): ...
    @classmethod
    def close_all(cls) -> None: ...
    @classmethod
    def identity_key(cls, *args, **kwargs): ...
    @classmethod
    def object_session(cls, instance): ...
    def __contains__(self, instance): ...
    def __iter__(self): ...
    def add(self, instance, _warn: bool = ...) -> None: ...
    def add_all(self, instances) -> None: ...
    def begin(self, subtransactions: bool = ..., nested: bool = ..., _subtrans: bool = ...): ...
    def begin_nested(self): ...
    def close(self) -> None: ...
    def commit(self) -> None: ...
    def connection(
        self, bind_arguments: Any | None = ..., close_with_result: bool = ..., execution_options: Any | None = ..., **kw
    ): ...
    def delete(self, instance) -> None: ...
    def execute(
        self,
        statement,
        params: Any | None = ...,
        execution_options=...,
        bind_arguments: Any | None = ...,
        _parent_execute_state: Any | None = ...,
        _add_event: Any | None = ...,
        **kw,
    ): ...
    def expire(self, instance, attribute_names: Any | None = ...) -> None: ...
    def expire_all(self) -> None: ...
    def expunge(self, instance) -> None: ...
    def expunge_all(self) -> None: ...
    def flush(self, objects: Any | None = ...) -> None: ...
    def get(
        self,
        entity,
        ident,
        options: Any | None = ...,
        populate_existing: bool = ...,
        with_for_update: Any | None = ...,
        identity_token: Any | None = ...,
    ): ...
    def get_bind(
        self,
        mapper: Any | None = ...,
        clause: Any | None = ...,
        bind: Any | None = ...,
        _sa_skip_events: Any | None = ...,
        _sa_skip_for_implicit_returning: bool = ...,
    ): ...
    def is_modified(self, instance, include_collections: bool = ...): ...
    def bulk_save_objects(
        self, objects, return_defaults: bool = ..., update_changed_only: bool = ..., preserve_order: bool = ...
    ): ...
    def bulk_insert_mappings(self, mapper, mappings, return_defaults: bool = ..., render_nulls: bool = ...) -> None: ...
    def bulk_update_mappings(self, mapper, mappings) -> None: ...
    def merge(self, instance, load: bool = ..., options: Any | None = ...): ...
    def query(self, *entities, **kwargs): ...
    def refresh(self, instance, attribute_names: Any | None = ..., with_for_update: Any | None = ...) -> None: ...
    def rollback(self) -> None: ...
    def scalar(self, statement, params: Any | None = ..., execution_options=..., bind_arguments: Any | None = ..., **kw): ...
    def scalars(self, statement, params: Any | None = ..., execution_options=..., bind_arguments: Any | None = ..., **kw): ...

ScopedSession = scoped_session



from .sql.base import SchemaVisitor as SchemaVisitor
from .sql.ddl import (
    DDL as DDL,
    AddConstraint as AddConstraint,
    CreateColumn as CreateColumn,
    CreateIndex as CreateIndex,
    CreateSchema as CreateSchema,
    CreateSequence as CreateSequence,
    CreateTable as CreateTable,
    DDLBase as DDLBase,
    DDLElement as DDLElement,
    DropColumnComment as DropColumnComment,
    DropConstraint as DropConstraint,
    DropIndex as DropIndex,
    DropSchema as DropSchema,
    DropSequence as DropSequence,
    DropTable as DropTable,
    DropTableComment as DropTableComment,
    SetColumnComment as SetColumnComment,
    SetTableComment as SetTableComment,
    _CreateDropBase as _CreateDropBase,
    _DDLCompiles as _DDLCompiles,
    _DropView as _DropView,
    sort_tables as sort_tables,
    sort_tables_and_constraints as sort_tables_and_constraints,
)
from .sql.naming import conv as conv
from .sql.schema import (
    BLANK_SCHEMA as BLANK_SCHEMA,
    CheckConstraint as CheckConstraint,
    Column as Column,
    ColumnCollectionConstraint as ColumnCollectionConstraint,
    ColumnCollectionMixin as ColumnCollectionMixin,
    ColumnDefault as ColumnDefault,
    Computed as Computed,
    Constraint as Constraint,
    DefaultClause as DefaultClause,
    DefaultGenerator as DefaultGenerator,
    FetchedValue as FetchedValue,
    ForeignKey as ForeignKey,
    ForeignKeyConstraint as ForeignKeyConstraint,
    Identity as Identity,
    Index as Index,
    MetaData as MetaData,
    PrimaryKeyConstraint as PrimaryKeyConstraint,
    SchemaItem as SchemaItem,
    Sequence as Sequence,
    Table as Table,
    ThreadLocalMetaData as ThreadLocalMetaData,
    UniqueConstraint as UniqueConstraint,
)



from typing import Any

from . import elements

REQUIRED: Any

class _multiparam_column(elements.ColumnElement):
    index: Any
    key: Any
    original: Any
    default: Any
    type: Any
    def __init__(self, original, index) -> None: ...
    def compare(self, other, **kw) -> None: ...
    def __eq__(self, other): ...



class immutabledict:
    def __len__(self) -> int: ...
    def __getitem__(self, __item): ...
    def __iter__(self): ...
    def union(self, **kwargs): ...
    def merge_with(self, *args): ...
    def keys(self): ...
    def __contains__(self, __item): ...
    def items(self): ...
    def values(self): ...
    def get(self, __key, __default=...): ...
    def __reduce__(self): ...



from typing import Any

EMPTY_ANNOTATIONS: Any

class SupportsAnnotations: ...
class SupportsCloneAnnotations(SupportsAnnotations): ...
class SupportsWrappingAnnotations(SupportsAnnotations): ...

class Annotated:
    def __new__(cls, *args): ...
    __dict__: Any
    def __init__(self, element, values) -> None: ...
    def __reduce__(self): ...
    def __hash__(self): ...
    def __eq__(self, other): ...
    @property
    def entity_namespace(self): ...

annotated_classes: Any



from typing import Any

from ..util import HasMemoized, hybridmethod
from . import base

DEL_ATTR: Any

class ClassManager(HasMemoized, dict[Any, Any]):
    MANAGER_ATTR: Any
    STATE_ATTR: Any
    expired_attribute_loader: Any
    init_method: Any
    factory: Any
    mapper: Any
    declarative_scan: Any
    registry: Any
    @property
    def deferred_scalar_loader(self): ...
    @deferred_scalar_loader.setter
    def deferred_scalar_loader(self, obj) -> None: ...
    class_: Any
    info: Any
    new_init: Any
    local_attrs: Any
    originals: Any
    def __init__(self, class_) -> None: ...
    def __hash__(self): ...
    def __eq__(self, other): ...
    @property
    def is_mapped(self): ...
    # Will be overwritten when mapped
    # def mapper(self) -> None: ...
    def manage(self) -> None: ...
    @hybridmethod
    def manager_getter(self): ...
    @hybridmethod
    def state_getter(self): ...
    @hybridmethod
    def dict_getter(self): ...
    def instrument_attribute(self, key, inst, propagated: bool = ...) -> None: ...
    def subclass_managers(self, recursive) -> None: ...
    def post_configure_attribute(self, key) -> None: ...
    def uninstrument_attribute(self, key, propagated: bool = ...) -> None: ...
    def unregister(self) -> None: ...
    def install_descriptor(self, key, inst) -> None: ...
    def uninstall_descriptor(self, key) -> None: ...
    def install_member(self, key, implementation) -> None: ...
    def uninstall_member(self, key) -> None: ...
    def instrument_collection_class(self, key, collection_class): ...
    def initialize_collection(self, key, state, factory): ...
    def is_instrumented(self, key, search: bool = ...): ...
    def get_impl(self, key): ...
    @property
    def attributes(self): ...
    def new_instance(self, state: Any | None = ...): ...
    def setup_instance(self, instance, state: Any | None = ...) -> None: ...
    def teardown_instance(self, instance) -> None: ...
    def has_state(self, instance): ...
    def has_parent(self, state, key, optimistic: bool = ...): ...
    def __bool__(self): ...
    __nonzero__: Any

class _SerializeManager:
    class_: Any
    def __init__(self, state, d) -> None: ...
    def __call__(self, state, inst, state_dict) -> None: ...

class InstrumentationFactory:
    def create_manager_for_cls(self, class_): ...
    def unregister(self, class_) -> None: ...

instance_state: Any

instance_dict: Any
manager_of_class = base.manager_of_class

def register_class(
    class_,
    finalize: bool = ...,
    mapper: Any | None = ...,
    registry: Any | None = ...,
    declarative_scan: Any | None = ...,
    expired_attribute_loader: Any | None = ...,
    init_method: Any | None = ...,
): ...
def unregister_class(class_) -> None: ...
def is_instrumented(instance, key): ...



def instrument_declarative(cls, cls_registry, metadata) -> None: ...

class ConcreteBase:
    @classmethod
    def __declare_first__(cls) -> None: ...

class AbstractConcreteBase(ConcreteBase):
    __no_table__: bool
    @classmethod
    def __declare_first__(cls) -> None: ...

class DeferredReflection:
    @classmethod
    def prepare(cls, engine) -> None: ...



from typing import Any

from ..sql.base import CompileState, Options
from ..sql.dml import DeleteDMLState, UpdateDMLState

def save_obj(base_mapper, states, uowtransaction, single: bool = ...) -> None: ...
def post_update(base_mapper, states, uowtransaction, post_update_cols) -> None: ...
def delete_obj(base_mapper, states, uowtransaction) -> None: ...

class BulkUDCompileState(CompileState):
    class default_update_options(Options): ...
    @classmethod
    def orm_pre_session_exec(cls, session, statement, params, execution_options, bind_arguments, is_reentrant_invoke): ...
    @classmethod
    def orm_setup_cursor_result(cls, session, statement, params, execution_options, bind_arguments, result): ...

class BulkORMUpdate(UpdateDMLState, BulkUDCompileState):
    mapper: Any
    extra_criteria_entities: Any
    @classmethod
    def create_for_statement(cls, statement, compiler, **kw): ...

class BulkORMDelete(DeleteDMLState, BulkUDCompileState):
    mapper: Any
    extra_criteria_entities: Any
    @classmethod
    def create_for_statement(cls, statement, compiler, **kw): ...



from typing import Any

from .base import BIT, MySQLDialect, MySQLExecutionContext

class _oursqlBIT(BIT):
    def result_processor(self, dialect, coltype) -> None: ...

class MySQLExecutionContext_oursql(MySQLExecutionContext):
    @property
    def plain_query(self): ...

class MySQLDialect_oursql(MySQLDialect):
    driver: str
    supports_statement_cache: bool
    supports_unicode_binds: bool
    supports_unicode_statements: bool
    supports_native_decimal: bool
    supports_sane_rowcount: bool
    supports_sane_multi_rowcount: bool
    colspecs: Any
    @classmethod
    def dbapi(cls): ...
    def do_execute(self, cursor, statement, parameters, context: Any | None = ...) -> None: ...
    def do_begin(self, connection) -> None: ...
    def do_begin_twophase(self, connection, xid) -> None: ...
    def do_prepare_twophase(self, connection, xid) -> None: ...
    def do_rollback_twophase(self, connection, xid, is_prepared: bool = ..., recover: bool = ...) -> None: ...
    def do_commit_twophase(self, connection, xid, is_prepared: bool = ..., recover: bool = ...) -> None: ...
    def has_table(self, connection, table_name, schema: Any | None = ...): ...  # type: ignore[override]
    def get_table_options(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_columns(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_view_names(self, connection, schema: Any | None = ..., **kw): ...
    def get_table_names(self, connection, schema: Any | None = ..., **kw): ...
    def get_schema_names(self, connection, **kw): ...
    def initialize(self, connection): ...
    def is_disconnect(self, e, connection, cursor): ...
    def create_connect_args(self, url): ...

dialect = MySQLDialect_oursql



from typing import Any

from ...engine import AdaptedConnection
from ...sql import sqltypes
from . import json
from .base import ENUM, INTERVAL, OID, REGCLASS, UUID, PGCompiler, PGDialect, PGExecutionContext, PGIdentifierPreparer

class AsyncpgTime(sqltypes.Time):
    def get_dbapi_type(self, dbapi): ...

class AsyncpgDate(sqltypes.Date):
    def get_dbapi_type(self, dbapi): ...

class AsyncpgDateTime(sqltypes.DateTime):
    def get_dbapi_type(self, dbapi): ...

class AsyncpgBoolean(sqltypes.Boolean):
    def get_dbapi_type(self, dbapi): ...

class AsyncPgInterval(INTERVAL):
    def get_dbapi_type(self, dbapi): ...
    @classmethod
    def adapt_emulated_to_native(cls, interval, **kw): ...

class AsyncPgEnum(ENUM):
    def get_dbapi_type(self, dbapi): ...

class AsyncpgInteger(sqltypes.Integer):
    def get_dbapi_type(self, dbapi): ...

class AsyncpgBigInteger(sqltypes.BigInteger):
    def get_dbapi_type(self, dbapi): ...

class AsyncpgJSON(json.JSON):
    def get_dbapi_type(self, dbapi): ...
    def result_processor(self, dialect, coltype) -> None: ...

class AsyncpgJSONB(json.JSONB):
    def get_dbapi_type(self, dbapi): ...
    def result_processor(self, dialect, coltype) -> None: ...

class AsyncpgJSONIndexType(sqltypes.JSON.JSONIndexType):
    def get_dbapi_type(self, dbapi) -> None: ...

class AsyncpgJSONIntIndexType(sqltypes.JSON.JSONIntIndexType):
    def get_dbapi_type(self, dbapi): ...

class AsyncpgJSONStrIndexType(sqltypes.JSON.JSONStrIndexType):
    def get_dbapi_type(self, dbapi): ...

class AsyncpgJSONPathType(json.JSONPathType):
    def bind_processor(self, dialect): ...

class AsyncpgUUID(UUID):
    def get_dbapi_type(self, dbapi): ...
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class AsyncpgNumeric(sqltypes.Numeric):
    def get_dbapi_type(self, dbapi): ...
    def bind_processor(self, dialect) -> None: ...
    def result_processor(self, dialect, coltype): ...

class AsyncpgFloat(AsyncpgNumeric):
    def get_dbapi_type(self, dbapi): ...

class AsyncpgREGCLASS(REGCLASS):
    def get_dbapi_type(self, dbapi): ...

class AsyncpgOID(OID):
    def get_dbapi_type(self, dbapi): ...

class PGExecutionContext_asyncpg(PGExecutionContext):
    def handle_dbapi_exception(self, e) -> None: ...
    exclude_set_input_sizes: Any
    def pre_exec(self) -> None: ...
    def create_server_side_cursor(self): ...

class PGCompiler_asyncpg(PGCompiler): ...
class PGIdentifierPreparer_asyncpg(PGIdentifierPreparer): ...

class AsyncAdapt_asyncpg_cursor:
    server_side: bool
    description: Any
    arraysize: int
    rowcount: int
    def __init__(self, adapt_connection) -> None: ...
    def close(self) -> None: ...
    def execute(self, operation, parameters: Any | None = ...) -> None: ...
    def executemany(self, operation, seq_of_parameters): ...
    def setinputsizes(self, *inputsizes) -> None: ...
    def __iter__(self): ...
    def fetchone(self): ...
    def fetchmany(self, size: Any | None = ...): ...
    def fetchall(self): ...

class AsyncAdapt_asyncpg_ss_cursor(AsyncAdapt_asyncpg_cursor):
    server_side: bool
    def __init__(self, adapt_connection) -> None: ...
    def close(self) -> None: ...
    def __aiter__(self): ...
    async def __anext__(self) -> None: ...
    def fetchone(self): ...
    def fetchmany(self, size: Any | None = ...): ...
    def fetchall(self): ...
    def executemany(self, operation, seq_of_parameters) -> None: ...

class AsyncAdapt_asyncpg_connection(AdaptedConnection):
    await_: Any
    dbapi: Any
    isolation_level: str
    readonly: bool
    deferrable: bool
    def __init__(self, dbapi, connection, prepared_statement_cache_size: int = ...) -> None: ...
    @property
    def autocommit(self): ...
    @autocommit.setter
    def autocommit(self, value) -> None: ...
    def set_isolation_level(self, level) -> None: ...
    def cursor(self, server_side: bool = ...): ...
    def rollback(self) -> None: ...
    def commit(self) -> None: ...
    def close(self) -> None: ...

class AsyncAdaptFallback_asyncpg_connection(AsyncAdapt_asyncpg_connection):
    await_: Any

class AsyncAdapt_asyncpg_dbapi:
    asyncpg: Any
    paramstyle: str
    def __init__(self, asyncpg) -> None: ...
    def connect(self, *arg, **kw): ...
    class Error(Exception): ...
    class Warning(Exception): ...
    class InterfaceError(Error): ...
    class DatabaseError(Error): ...
    class InternalError(DatabaseError): ...
    class OperationalError(DatabaseError): ...
    class ProgrammingError(DatabaseError): ...
    class IntegrityError(DatabaseError): ...
    class DataError(DatabaseError): ...
    class NotSupportedError(DatabaseError): ...
    class InternalServerError(InternalError): ...
    class InvalidCachedStatementError(NotSupportedError):
        def __init__(self, message) -> None: ...
    def Binary(self, value): ...
    STRING: Any
    TIMESTAMP: Any
    TIMESTAMP_W_TZ: Any
    TIME: Any
    DATE: Any
    INTERVAL: Any
    NUMBER: Any
    FLOAT: Any
    BOOLEAN: Any
    INTEGER: Any
    BIGINTEGER: Any
    BYTES: Any
    DECIMAL: Any
    JSON: Any
    JSONB: Any
    ENUM: Any
    UUID: Any
    BYTEA: Any
    DATETIME: Any
    BINARY: Any

class PGDialect_asyncpg(PGDialect):
    driver: str
    supports_statement_cache: bool
    supports_unicode_statements: bool
    supports_server_side_cursors: bool
    supports_unicode_binds: bool
    default_paramstyle: str
    supports_sane_multi_rowcount: bool
    statement_compiler: Any
    preparer: Any
    use_setinputsizes: bool
    use_native_uuid: bool
    colspecs: Any
    is_async: bool
    @classmethod
    def dbapi(cls): ...
    def set_isolation_level(self, connection, level) -> None: ...
    def set_readonly(self, connection, value) -> None: ...
    def get_readonly(self, connection): ...
    def set_deferrable(self, connection, value) -> None: ...
    def get_deferrable(self, connection): ...
    def create_connect_args(self, url): ...
    @classmethod
    def get_pool_class(cls, url): ...
    def is_disconnect(self, e, connection, cursor): ...
    def do_set_input_sizes(self, cursor, list_of_tuples, context) -> None: ...
    async def setup_asyncpg_json_codec(self, conn): ...
    async def setup_asyncpg_jsonb_codec(self, conn): ...
    def on_connect(self): ...
    def get_driver_connection(self, connection): ...

dialect = PGDialect_asyncpg



from collections.abc import KeysView
from typing import Any

from ..sql.base import InPlaceGenerative

class ResultMetaData:
    @property
    def keys(self): ...

class RMKeyView(KeysView[Any]):
    def __init__(self, parent) -> None: ...
    def __len__(self): ...
    def __iter__(self): ...
    def __contains__(self, item): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...

class SimpleResultMetaData(ResultMetaData):
    def __init__(
        self,
        keys,
        extra: Any | None = ...,
        _processors: Any | None = ...,
        _tuplefilter: Any | None = ...,
        _translated_indexes: Any | None = ...,
        _unique_filters: Any | None = ...,
    ) -> None: ...

def result_tuple(fields, extra: Any | None = ...): ...

class ResultInternal(InPlaceGenerative): ...

class _WithKeys:
    def keys(self): ...

class Result(_WithKeys, ResultInternal):
    def __init__(self, cursor_metadata) -> None: ...
    def close(self) -> None: ...
    def yield_per(self, num) -> None: ...
    def unique(self, strategy: Any | None = ...) -> None: ...
    def columns(self, *col_expressions): ...
    def scalars(self, index: int = ...): ...
    def mappings(self): ...
    def __iter__(self): ...
    def __next__(self): ...
    def partitions(self, size: Any | None = ...) -> None: ...
    def fetchall(self): ...
    def fetchone(self): ...
    def fetchmany(self, size: Any | None = ...): ...
    def all(self): ...
    def first(self): ...
    def one_or_none(self): ...
    def scalar_one(self): ...
    def scalar_one_or_none(self): ...
    def one(self): ...
    def scalar(self): ...
    def freeze(self): ...
    def merge(self, *others): ...

class FilterResult(ResultInternal): ...

class ScalarResult(FilterResult):
    def __init__(self, real_result, index) -> None: ...
    def unique(self, strategy: Any | None = ...): ...
    def partitions(self, size: Any | None = ...) -> None: ...
    def fetchall(self): ...
    def fetchmany(self, size: Any | None = ...): ...
    def all(self): ...
    def __iter__(self): ...
    def __next__(self): ...
    def first(self): ...
    def one_or_none(self): ...
    def one(self): ...

class MappingResult(_WithKeys, FilterResult):
    def __init__(self, result) -> None: ...
    def unique(self, strategy: Any | None = ...): ...
    def columns(self, *col_expressions): ...
    def partitions(self, size: Any | None = ...) -> None: ...
    def fetchall(self): ...
    def fetchone(self): ...
    def fetchmany(self, size: Any | None = ...): ...
    def all(self): ...
    def __iter__(self): ...
    def __next__(self): ...
    def first(self): ...
    def one_or_none(self): ...
    def one(self): ...

class FrozenResult:
    metadata: Any
    data: Any
    def __init__(self, result) -> None: ...
    def rewrite_rows(self): ...
    def with_new_rows(self, tuple_data): ...
    def __call__(self): ...

class IteratorResult(Result):
    iterator: Any
    raw: Any
    def __init__(self, cursor_metadata, iterator, raw: Any | None = ..., _source_supports_scalars: bool = ...) -> None: ...

def null_result(): ...

class ChunkedIteratorResult(IteratorResult):
    chunks: Any
    raw: Any
    iterator: Any
    dynamic_yield_per: Any
    def __init__(
        self, cursor_metadata, chunks, source_supports_scalars: bool = ..., raw: Any | None = ..., dynamic_yield_per: bool = ...
    ) -> None: ...
    def yield_per(self, num) -> None: ...

class MergedResult(IteratorResult):
    closed: bool
    def __init__(self, cursor_metadata, results) -> None: ...



from typing import Any

def compiles(class_, *specs): ...
def deregister(class_) -> None: ...

class _dispatcher:
    specs: Any
    def __init__(self) -> None: ...
    def __call__(self, element, compiler, **kw): ...



import asyncio as asyncio
from collections.abc import Callable, Coroutine
from typing import Any

from .langhelpers import memoized_property

_greenlet = Any  # actually greenlet.greenlet

def is_exit_exception(e): ...

class _AsyncIoGreenlet(_greenlet):
    driver: Any
    gr_context: Any
    def __init__(self, fn, driver) -> None: ...

def await_only(awaitable: Coroutine[Any, Any, Any]) -> Any: ...
def await_fallback(awaitable: Coroutine[Any, Any, Any]) -> Any: ...
async def greenlet_spawn(fn: Callable[..., Any], *args, _require_await: bool = ..., **kwargs) -> Any: ...

class AsyncAdaptedLock:
    @memoized_property
    def mutex(self): ...
    def __enter__(self): ...
    def __exit__(self, *arg, **kw) -> None: ...

def get_event_loop(): ...



from typing import Any

class Empty(Exception): ...
class Full(Exception): ...

class Queue:
    mutex: Any
    not_empty: Any
    not_full: Any
    use_lifo: Any
    def __init__(self, maxsize: int = ..., use_lifo: bool = ...) -> None: ...
    def qsize(self): ...
    def empty(self): ...
    def full(self): ...
    def put(self, item, block: bool = ..., timeout: Any | None = ...) -> None: ...
    def put_nowait(self, item): ...
    def get(self, block: bool = ..., timeout: Any | None = ...): ...
    def get_nowait(self): ...

class AsyncAdaptedQueue:
    await_: Any
    use_lifo: Any
    maxsize: Any
    def __init__(self, maxsize: int = ..., use_lifo: bool = ...) -> None: ...
    def empty(self): ...
    def full(self): ...
    def qsize(self): ...
    def put_nowait(self, item): ...
    def put(self, item, block: bool = ..., timeout: Any | None = ...): ...
    def get_nowait(self): ...
    def get(self, block: bool = ..., timeout: Any | None = ...): ...

class FallbackAsyncAdaptedQueue(AsyncAdaptedQueue):
    await_: Any



from .engine import (
    create_engine as create_engine,
    create_mock_engine as create_mock_engine,
    engine_from_config as engine_from_config,
)
from .inspection import inspect as inspect
from .schema import (
    BLANK_SCHEMA as BLANK_SCHEMA,
    DDL as DDL,
    CheckConstraint as CheckConstraint,
    Column as Column,
    ColumnDefault as ColumnDefault,
    Computed as Computed,
    Constraint as Constraint,
    DefaultClause as DefaultClause,
    FetchedValue as FetchedValue,
    ForeignKey as ForeignKey,
    ForeignKeyConstraint as ForeignKeyConstraint,
    Identity as Identity,
    Index as Index,
    MetaData as MetaData,
    PrimaryKeyConstraint as PrimaryKeyConstraint,
    Sequence as Sequence,
    Table as Table,
    ThreadLocalMetaData as ThreadLocalMetaData,
    UniqueConstraint as UniqueConstraint,
)
from .sql import (
    LABEL_STYLE_DEFAULT as LABEL_STYLE_DEFAULT,
    LABEL_STYLE_DISAMBIGUATE_ONLY as LABEL_STYLE_DISAMBIGUATE_ONLY,
    LABEL_STYLE_NONE as LABEL_STYLE_NONE,
    LABEL_STYLE_TABLENAME_PLUS_COL as LABEL_STYLE_TABLENAME_PLUS_COL,
    alias as alias,
    all_ as all_,
    and_ as and_,
    any_ as any_,
    asc as asc,
    between as between,
    bindparam as bindparam,
    case as case,
    cast as cast,
    collate as collate,
    column as column,
    delete as delete,
    desc as desc,
    distinct as distinct,
    except_ as except_,
    except_all as except_all,
    exists as exists,
    extract as extract,
    false as false,
    func as func,
    funcfilter as funcfilter,
    insert as insert,
    intersect as intersect,
    intersect_all as intersect_all,
    join as join,
    lambda_stmt as lambda_stmt,
    lateral as lateral,
    literal as literal,
    literal_column as literal_column,
    modifier as modifier,
    not_ as not_,
    null as null,
    nulls_first as nulls_first,
    nulls_last as nulls_last,
    nullsfirst as nullsfirst,
    nullslast as nullslast,
    or_ as or_,
    outerjoin as outerjoin,
    outparam as outparam,
    over as over,
    select as select,
    subquery as subquery,
    table as table,
    tablesample as tablesample,
    text as text,
    true as true,
    tuple_ as tuple_,
    type_coerce as type_coerce,
    union as union,
    union_all as union_all,
    update as update,
    values as values,
    within_group as within_group,
)
from .types import (
    ARRAY as ARRAY,
    BIGINT as BIGINT,
    BINARY as BINARY,
    BLOB as BLOB,
    BOOLEAN as BOOLEAN,
    CHAR as CHAR,
    CLOB as CLOB,
    DATE as DATE,
    DATETIME as DATETIME,
    DECIMAL as DECIMAL,
    FLOAT as FLOAT,
    INT as INT,
    INTEGER as INTEGER,
    JSON as JSON,
    NCHAR as NCHAR,
    NUMERIC as NUMERIC,
    NVARCHAR as NVARCHAR,
    REAL as REAL,
    SMALLINT as SMALLINT,
    TEXT as TEXT,
    TIME as TIME,
    TIMESTAMP as TIMESTAMP,
    VARBINARY as VARBINARY,
    VARCHAR as VARCHAR,
    BigInteger as BigInteger,
    Boolean as Boolean,
    Date as Date,
    DateTime as DateTime,
    Enum as Enum,
    Float as Float,
    Integer as Integer,
    Interval as Interval,
    LargeBinary as LargeBinary,
    Numeric as Numeric,
    PickleType as PickleType,
    SmallInteger as SmallInteger,
    String as String,
    Text as Text,
    Time as Time,
    TupleType as TupleType,
    TypeDecorator as TypeDecorator,
    Unicode as Unicode,
    UnicodeText as UnicodeText,
)

__version__: str



from typing import Any

from ..orm import interfaces

HYBRID_METHOD: Any
HYBRID_PROPERTY: Any

class hybrid_method(interfaces.InspectionAttrInfo):
    is_attribute: bool
    extension_type: Any
    func: Any
    def __init__(self, func, expr: Any | None = ...) -> None: ...
    def __get__(self, instance, owner): ...
    expr: Any
    def expression(self, expr): ...

class hybrid_property(interfaces.InspectionAttrInfo):
    is_attribute: bool
    extension_type: Any
    fget: Any
    fset: Any
    fdel: Any
    expr: Any
    custom_comparator: Any
    update_expr: Any
    def __init__(
        self,
        fget,
        fset: Any | None = ...,
        fdel: Any | None = ...,
        expr: Any | None = ...,
        custom_comparator: Any | None = ...,
        update_expr: Any | None = ...,
    ) -> None: ...
    def __get__(self, instance, owner): ...
    def __set__(self, instance, value) -> None: ...
    def __delete__(self, instance) -> None: ...
    @property
    def overrides(self): ...
    def getter(self, fget): ...
    def setter(self, fset): ...
    def deleter(self, fdel): ...
    def expression(self, expr): ...
    def comparator(self, comparator): ...
    def update_expression(self, meth): ...

class Comparator(interfaces.PropComparator):
    property: Any
    expression: Any
    def __init__(self, expression) -> None: ...
    def __clause_element__(self): ...
    def adapt_to_entity(self, adapt_to_entity): ...

_property = property

class ExprComparator(Comparator):
    cls: Any
    expression: Any
    hybrid: Any
    def __init__(self, cls, expression, hybrid) -> None: ...
    def __getattr__(self, key): ...
    @_property
    def info(self): ...
    @_property
    def property(self): ...
    def operate(self, op, *other, **kwargs): ...
    def reverse_operate(self, op, other, **kwargs): ...



from typing import Any

from ..sql.compiler import Compiled as Compiled, TypeCompiler as TypeCompiler

class Dialect:
    supports_statement_cache: bool
    def create_connect_args(self, url) -> None: ...
    @classmethod
    def type_descriptor(cls, typeobj) -> None: ...
    def initialize(self, connection) -> None: ...
    def get_columns(self, connection, table_name, schema: Any | None = ..., **kw) -> None: ...
    def get_pk_constraint(self, connection, table_name, schema: Any | None = ..., **kw) -> None: ...
    def get_foreign_keys(self, connection, table_name, schema: Any | None = ..., **kw) -> None: ...
    def get_table_names(self, connection, schema: Any | None = ..., **kw) -> None: ...
    def get_temp_table_names(self, connection, schema: Any | None = ..., **kw) -> None: ...
    def get_view_names(self, connection, schema: Any | None = ..., **kw) -> None: ...
    def get_sequence_names(self, connection, schema: Any | None = ..., **kw) -> None: ...
    def get_temp_view_names(self, connection, schema: Any | None = ..., **kw) -> None: ...
    def get_view_definition(self, connection, view_name, schema: Any | None = ..., **kw) -> None: ...
    def get_indexes(self, connection, table_name, schema: Any | None = ..., **kw) -> None: ...
    def get_unique_constraints(self, connection, table_name, schema: Any | None = ..., **kw) -> None: ...
    def get_check_constraints(self, connection, table_name, schema: Any | None = ..., **kw) -> None: ...
    def get_table_comment(self, connection, table_name, schema: Any | None = ..., **kw) -> None: ...
    def normalize_name(self, name) -> None: ...
    def denormalize_name(self, name) -> None: ...
    def has_table(self, connection, table_name, schema: Any | None = ..., **kw) -> None: ...
    def has_index(self, connection, table_name, index_name, schema: Any | None = ...) -> None: ...
    def has_sequence(self, connection, sequence_name, schema: Any | None = ..., **kw) -> None: ...
    def do_begin(self, dbapi_connection) -> None: ...
    def do_rollback(self, dbapi_connection) -> None: ...
    def do_commit(self, dbapi_connection) -> None: ...
    def do_close(self, dbapi_connection) -> None: ...
    def do_set_input_sizes(self, cursor, list_of_tuples, context) -> None: ...
    def create_xid(self) -> None: ...
    def do_savepoint(self, connection, name) -> None: ...
    def do_rollback_to_savepoint(self, connection, name) -> None: ...
    def do_release_savepoint(self, connection, name) -> None: ...
    def do_begin_twophase(self, connection, xid) -> None: ...
    def do_prepare_twophase(self, connection, xid) -> None: ...
    def do_rollback_twophase(self, connection, xid, is_prepared: bool = ..., recover: bool = ...) -> None: ...
    def do_commit_twophase(self, connection, xid, is_prepared: bool = ..., recover: bool = ...) -> None: ...
    def do_recover_twophase(self, connection) -> None: ...
    def do_executemany(self, cursor, statement, parameters, context: Any | None = ...) -> None: ...
    def do_execute(self, cursor, statement, parameters, context: Any | None = ...) -> None: ...
    def do_execute_no_params(self, cursor, statement, parameters, context: Any | None = ...) -> None: ...
    def is_disconnect(self, e, connection, cursor) -> None: ...
    def connect(self, *cargs, **cparams) -> None: ...
    def on_connect_url(self, url): ...
    def on_connect(self) -> None: ...
    def reset_isolation_level(self, dbapi_conn) -> None: ...
    def set_isolation_level(self, dbapi_conn, level) -> None: ...
    def get_isolation_level(self, dbapi_conn) -> None: ...
    def get_default_isolation_level(self, dbapi_conn) -> None: ...
    @classmethod
    def get_dialect_cls(cls, url): ...
    @classmethod
    def load_provisioning(cls) -> None: ...
    @classmethod
    def engine_created(cls, engine) -> None: ...
    def get_driver_connection(self, connection) -> None: ...

class CreateEnginePlugin:
    url: Any
    def __init__(self, url, kwargs) -> None: ...
    def update_url(self, url) -> None: ...
    def handle_dialect_kwargs(self, dialect_cls, dialect_args) -> None: ...
    def handle_pool_kwargs(self, pool_cls, pool_args) -> None: ...
    def engine_created(self, engine) -> None: ...

class ExecutionContext:
    def create_cursor(self) -> None: ...
    def pre_exec(self) -> None: ...
    def get_out_parameter_values(self, out_param_names) -> None: ...
    def post_exec(self) -> None: ...
    def get_result_cursor_strategy(self, result) -> None: ...
    def handle_dbapi_exception(self, e) -> None: ...
    def should_autocommit_text(self, statement) -> None: ...
    def lastrow_has_defaults(self) -> None: ...
    def get_rowcount(self) -> None: ...

class Connectable:
    def connect(self, **kwargs) -> None: ...
    engine: Any
    def execute(self, object_, *multiparams, **params) -> None: ...
    def scalar(self, object_, *multiparams, **params) -> None: ...

class ExceptionContext:
    connection: Any
    engine: Any
    cursor: Any
    statement: Any
    parameters: Any
    original_exception: Any
    sqlalchemy_exception: Any
    chained_exception: Any
    execution_context: Any
    is_disconnect: Any
    invalidate_pool_on_disconnect: bool

class AdaptedConnection:
    @property
    def driver_connection(self): ...



from .. import exc as sa_exc

class SATestSuiteWarning(sa_exc.SAWarning): ...

def warn_test_suite(message) -> None: ...
def setup_filters() -> None: ...
def assert_warnings(fn, warning_msgs, regex: bool = ...): ...



from typing import Any

from ..orm.instrumentation import ClassManager, InstrumentationFactory

INSTRUMENTATION_MANAGER: str

def find_native_user_instrumentation_hook(cls): ...

instrumentation_finders: Any

class ExtendedInstrumentationRegistry(InstrumentationFactory):
    def unregister(self, class_) -> None: ...
    def manager_of_class(self, cls): ...
    def state_of(self, instance): ...
    def dict_of(self, instance): ...

class InstrumentationManager:
    def __init__(self, class_) -> None: ...
    def manage(self, class_, manager) -> None: ...
    def unregister(self, class_, manager) -> None: ...
    def manager_getter(self, class_): ...
    def instrument_attribute(self, class_, key, inst) -> None: ...
    def post_configure_attribute(self, class_, key, inst) -> None: ...
    def install_descriptor(self, class_, key, inst) -> None: ...
    def uninstall_descriptor(self, class_, key) -> None: ...
    def install_member(self, class_, key, implementation) -> None: ...
    def uninstall_member(self, class_, key) -> None: ...
    def instrument_collection_class(self, class_, key, collection_class): ...
    def get_instance_dict(self, class_, instance): ...
    def initialize_instance_dict(self, class_, instance) -> None: ...
    def install_state(self, class_, instance, state) -> None: ...
    def remove_state(self, class_, instance) -> None: ...
    def state_getter(self, class_): ...
    def dict_getter(self, class_): ...

class _ClassInstrumentationAdapter(ClassManager):
    def __init__(self, class_, override) -> None: ...
    def manage(self) -> None: ...
    def unregister(self) -> None: ...
    def manager_getter(self): ...
    def instrument_attribute(self, key, inst, propagated: bool = ...) -> None: ...
    def post_configure_attribute(self, key) -> None: ...
    def install_descriptor(self, key, inst) -> None: ...
    def uninstall_descriptor(self, key) -> None: ...
    def install_member(self, key, implementation) -> None: ...
    def uninstall_member(self, key) -> None: ...
    def instrument_collection_class(self, key, collection_class): ...
    def initialize_collection(self, key, state, factory): ...
    def new_instance(self, state: Any | None = ...): ...
    def setup_instance(self, instance, state: Any | None = ...): ...
    def teardown_instance(self, instance) -> None: ...
    def has_state(self, instance): ...
    def state_getter(self): ...
    def dict_getter(self): ...



from ..dialects.firebird import base as firebird_base
from ..dialects.mssql import base as mssql_base
from ..dialects.mysql import base as mysql_base
from ..dialects.oracle import base as oracle_base
from ..dialects.postgresql import base as postgresql_base
from ..dialects.sqlite import base as sqlite_base
from ..dialects.sybase import base as sybase_base

__all__ = ("firebird", "mssql", "mysql", "postgresql", "sqlite", "oracle", "sybase")

firebird = firebird_base
mssql = mssql_base
mysql = mysql_base
oracle = oracle_base
postgresql = postgresql_base
postgres = postgresql_base
sqlite = sqlite_base
sybase = sybase_base
